'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.resolverConnection = resolverConnection;
exports.resolverList = resolverList;
exports.resolverOne = resolverOne;
exports.resolverCreate = resolverCreate;
exports.resolverUpdate = resolverUpdate;
exports.resolverDelete = resolverDelete;
exports.default = resolver;

var _graphql = require('graphql');

var _lodash = require('lodash');

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// - add exntesion to file
// - return path instead of {name, type, size, path}
function transformFile(args) {
  return (0, _lodash.cloneDeepWith)(args, v => {
    if ((0, _lodash.isObject)(v) && (0, _lodash.has)(v, 'name') && (0, _lodash.has)(v, 'type') && (0, _lodash.has)(v, 'size') && (0, _lodash.has)(v, 'path')) {
      // type: 'applciaton/png'
      const parts = v.type.split('/');
      const ext = parts[parts.length - 1];
      const newPath = `${v.path}.${ext}`;
      _fs2.default.renameSync(v.path, newPath);
      return _path2.default.basename(newPath);
    }
    return undefined;
  });
}

// Add get, set support
// processFields(Model, args, {get: true, set: true, transformFile: true })
function processFields(fields, args, options) {
  if (options.transformFile) {
    args = transformFile(args);
  }

  (0, _lodash.forOwn)(fields, (field, key) => {
    // posts { body }
    if (options.get) {
      if (field.get) {
        args[key] = field.get.apply(args, [args[key]]);
      }
    }

    // createPost({body: 'hello'})
    if (options.set) {
      if (field.set) {
        args[key] = field.set.apply(args, [args[key]]);
      }
    }
  });
  return args;
}

function resolverConnection(Model) {
  return (obj, args, context) => {
    const result = context.db.queryConnection(Model, args);
    result.nodes = result.nodes.map(v => processFields(Model.fields, v, { get: true }));
    return result;
  };
}

function resolverList(Model) {
  return (obj, args, context) => {
    return context.db.queryList(Model, args);
  };
}

function resolverOne(Model) {
  return (obj, args, context) => {
    const result = context.db.queryOne(Model, args);
    return processFields(Model.fields, result, { get: true });
  };
}

function resolverCreate(Model) {
  return async (obj, args, context) => {
    const input = processFields(Model.fields, args.input, {
      set: true,
      transformFile: true
    });
    Model.hooks.beforeCreate();
    const result = await context.db.mutationCreate(Model, input);
    Model.hooks.afterCreate();
    return result;
  };
}

function resolverUpdate(Model) {
  return async (obj, args, context) => {
    const input = processFields(Model.fields, args.input, {
      set: true,
      transformFile: true
    });
    Model.hooks.beforeUpdate();
    const result = await context.db.mutationUpdate(Model, args.id, input);
    Model.hooks.afterUpdate();
    return result;
  };
}

function resolverDelete(Model) {
  return async (obj, args, context) => {
    Model.hooks.beforeDelete();
    const result = await context.db.mutationDelete(Model, args.id);
    Model.hooks.afterDelete();
    return result;
  };
}

function resolver(Model) {
  return (obj, args, context, info) => {
    let fn;
    if (info.returnType.name && info.returnType.name.endsWith('Connection')) {
      fn = resolverConnection;
    } else if (info.fieldName.startsWith('create')) {
      fn = resolverCreate;
    } else if (info.fieldName.startsWith('update')) {
      fn = resolverUpdate;
    } else if (info.fieldName.startsWith('delete')) {
      fn = resolverDelete;
    } else if (info.returnType instanceof _graphql.GraphQLList) {
      fn = resolverList;
    } else {
      fn = resolverOne;
    }
    return fn(Model)(obj, args, context, info);
  };
}